//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

// This file isn't generated, but this comment is necessary to exclude it from StyleCop analysis.
// <auto-generated/>

using System;
using System.Linq;

using static System.Linq.Enumerable;
using static Microsoft.Data.Encryption.Resources.Strings;

namespace Microsoft.Data.Encryption.Cryptography.Serializers
{
    /// <summary>
    /// Contains the methods for serializing and deserializing <see cref="byte"/>[] type data objects
	/// that is compatible with the Always Encrypted feature in SQL Server and Azure SQL.
    /// </summary>
    internal sealed class SqlBinarySerializer : Serializer<byte[]>
    {
        private const int DefaultSize = 30;
        private const int MinSize = 1;
        private const int MaxSize = 8000;
        private static readonly byte Padding = 0;

        /// <summary>
        /// The <see cref="Identifier"/> uniquely identifies a particular Serializer implementation.
        /// </summary>
        public override string Identifier => "SQL_Binary_Nullable";

        private int size;

        /// <summary>
        /// Gets or sets the maximum length of the data.
        /// </summary>
        /// <remarks>
        /// If not explicitly set, the size is defaulted to 30.
        /// </remarks>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// Thrown when set to a value that is out of the valid range [1 - 8000] for this setting.
        /// </exception>
        public int Size
        {
            get => size;
            set
            {
                if (value < MinSize || value > MaxSize)
                {
                    throw new MicrosoftDataEncryptionException(ValueOutOfRange.Format(value));
                }

                size = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlBinarySerializer"/> class.
        /// </summary>
        /// <param name="size">The maximum length of the data</param>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// Thrown when set to a value that is out of the valid range [1 - 8000] for this setting.
        /// </exception>
        public SqlBinarySerializer(int size = DefaultSize)
        {
            Size = size;
        }

        /// <summary>
        /// Deserializes the provided <paramref name="bytes"/>
        /// </summary>
        /// <param name="bytes">The data to be deserialized</param>
        /// <returns>The serialized data</returns>
        /// <remarks>
        /// If the deserialized value is less than the size, then the value will be padded on the
        /// left to match the size. Padding is achieved by using hexadecimal zeros.
        /// </remarks>
        public override byte[] Deserialize(byte[] bytes)
        {
            return bytes.IsNull() ? null : PadToLength(bytes);
        }

        /// <summary>
        /// Serializes the provided <paramref name="value"/>
        /// </summary>
        /// <param name="value">The value to be serialized</param>
        /// <returns>The serialized data as a byte array</returns>
        /// <remarks>
        /// If the value is greater than the size, then the value will be truncated on the left to match the size.
        /// </remarks>
        public override byte[] Serialize(byte[] value)
        {
            return value.IsNull() ? null : TrimToLength(value);
        }

        private byte[] TrimToLength(byte[] value) => value.Length > Size ? value.Take(Size).ToArray() : value;

        private byte[] PadToLength(byte[] value) => value.Length < Size ? value.Concat(Repeat(Padding, Size - value.Length)).ToArray() : value;
    }
}
